@@ -1,6 +1,7 @@
 package Statocles::App::Blog;
+$Statocles::App::Blog::VERSION = '0.071';
 # ABSTRACT: A blog application
-$Statocles::App::Blog::VERSION = '0.070';
+
 use Statocles::Base 'Class';
 use Getopt::Long qw( GetOptionsFromArray );
 use Statocles::Store;
@@ -10,7 +11,57 @@ use Statocles::Util qw( run_editor );
 
 with 'Statocles::App::Role::Store';
 
+#pod =attr store
+#pod
+#pod     # site.yml
+#pod     blog:
+#pod         class: Statocles::App::Blog
+#pod         args:
+#pod             store: _posts
+#pod
+#pod The L<store directory path|Statocles::Store> to read for blog posts. Required.
+#pod
+#pod The Blog directory is organized in a tree by date, with a directory for the
+#pod year, month, day, and post. Each blog post is its own directory to allow for
+#pod additional files for the post, like images or additional pages.
+#pod
+#pod =cut
 
+#pod =attr tag_text
+#pod
+#pod     # site.yml
+#pod     blog:
+#pod         class: Statocles::App::Blog
+#pod         args:
+#pod             tag_text:
+#pod                 software: Posts about software and development
+#pod                 travel: My travelogue around the world!
+#pod
+#pod A hash of tag and introductory Markdown that will be shown on the tag's main
+#pod page. Having a description is optional.
+#pod
+#pod Using L<Beam::Wire's $config directive|Beam::Wire/Config Services>, you can
+#pod put the tag text in an external file:
+#pod
+#pod     # site.yml
+#pod     blog:
+#pod         class: Statocles::App::Blog
+#pod         args:
+#pod             tag_text:
+#pod                 $config: tags.yml
+#pod
+#pod     # tags.yml
+#pod     software: |-
+#pod         # Software
+#pod
+#pod         Posts about software development, mostly in [Perl](http://perl.org)
+#pod
+#pod     travel: |-
+#pod         # Travel
+#pod
+#pod         My travelogue around the world! [Also visit my Instagram!](http://example.com)
+#pod
+#pod =cut
 
 has tag_text => (
     is => 'ro',
@@ -18,6 +69,18 @@ has tag_text => (
     default => sub { {} },
 );
 
+#pod =attr page_size
+#pod
+#pod     # site.yml
+#pod     blog:
+#pod         class: Statocles::App::Blog
+#pod         args:
+#pod             page_size: 5
+#pod
+#pod The number of posts to put in a page (the main page and the tag pages). Defaults
+#pod to 5.
+#pod
+#pod =cut
 
 has page_size => (
     is => 'ro',
@@ -25,6 +88,27 @@ has page_size => (
     default => sub { 5 },
 );
 
+#pod =attr index_tags
+#pod
+#pod     # site.yml
+#pod     blog:
+#pod         class: Statocles::App::Blog
+#pod         args:
+#pod             index_tags: [ '-private', '+important' ]
+#pod
+#pod Filter the tags shown in the index page. An array of tags prefixed with either
+#pod a + or a -. By prefixing the tag with a "-", it will be removed from the index,
+#pod unless a later tag prefixed with a "+" also matches.
+#pod
+#pod By default, all tags are shown on the index page.
+#pod
+#pod So, given a document with tags "foo", and "bar":
+#pod
+#pod     index_tags: [ ]                 # document will be included
+#pod     index_tags: [ '-foo' ]          # document will not be included
+#pod     index_tags: [ '-foo', '+bar' ]  # document will be included
+#pod
+#pod =cut
 
 has index_tags => (
     is => 'ro',
@@ -32,6 +116,12 @@ has index_tags => (
     default => sub { [] },
 );
 
+#pod =attr template_dir
+#pod
+#pod The directory (inside the theme directory) to use for this app's templates.
+#pod Defaults to C<blog>.
+#pod
+#pod =cut
 
 has '+template_dir' => (
     default => 'blog',
@@ -45,6 +135,14 @@ has _post_pages => (
     predicate => '_has_cached_post_pages',
 );
 
+#pod =method command
+#pod
+#pod     my $exitval = $app->command( $app_name, @args );
+#pod
+#pod Run a command on this app. The app name is used to build the help, so
+#pod users get exactly what they need to run.
+#pod
+#pod =cut
 
 our $default_post = {
     tags => undef,
@@ -155,6 +253,13 @@ ENDHELP
     return 0;
 }
 
+#pod =method make_slug
+#pod
+#pod     my $slug = $app->make_slug( $title );
+#pod
+#pod Given a post title, remove special characters to create a slug.
+#pod
+#pod =cut
 
 sub make_slug {
     my ( $self, $slug ) = @_;
@@ -162,6 +267,14 @@ sub make_slug {
     return lc $slug;
 }
 
+#pod =method index
+#pod
+#pod     my @pages = $app->index( \@post_pages );
+#pod
+#pod Build the index page (a L<list page|Statocles::Page::List>) and all related
+#pod feed pages out of the given array reference of post pages.
+#pod
+#pod =cut
 
 my %FEEDS = (
     rss => {
@@ -177,18 +290,25 @@ my %FEEDS = (
 sub index {
     my ( $self, $all_post_pages ) = @_;
 
-    # Filter the index_tags
+    my @index_tags;
+    my %tag_flag;
+    for my $tag_spec ( @{ $self->index_tags } ) {
+        my $tag = substr $tag_spec, 1;
+        push @index_tags, $tag;
+        $tag_flag{$tag} = substr $tag_spec, 0, 1;
+    }
+
     my @index_post_pages;
     PAGE: for my $page ( @$all_post_pages ) {
-        my $add = 1;
-        for my $tag_spec ( @{ $self->index_tags } ) {
-            my $flag = substr $tag_spec, 0, 1;
-            my $tag = substr $tag_spec, 1;
-            if ( grep { $_ eq $tag } @{ $page->document->tags } ) {
-                $add = $flag eq '-' ? 0 : 1;
+        my $page_flag = '+';
+        my %page_tags;
+        @page_tags{ @{ $page->document->tags } } = 1; # we use exists(), so value doesn't matter
+        for my $tag ( @index_tags ) {
+            if ( exists $page_tags{ $tag } ) {
+                $page_flag = $tag_flag{ $tag };
             }
         }
-        push @index_post_pages, $page if $add;
+        push @index_post_pages, $page if $page_flag eq '+';
     }
 
     my @pages = Statocles::Page::List->paginate(
@@ -240,6 +360,15 @@ sub index {
     return ( @pages, @feed_pages );
 }
 
+#pod =method tag_pages
+#pod
+#pod     my @pages = $app->tag_pages( \%tag_pages );
+#pod
+#pod Get L<pages|Statocles::Page> for the tags in the given blog post documents
+#pod (build from L<the post_pages method|/post_pages>, including relevant feed
+#pod pages.
+#pod
+#pod =cut
 
 sub tag_pages {
     my ( $self, $tagged_docs ) = @_;
@@ -301,6 +430,23 @@ sub tag_pages {
     return @pages;
 }
 
+#pod =method pages
+#pod
+#pod     my @pages = $app->pages( %options );
+#pod
+#pod Get all the L<pages|Statocles::Page> for this application. Available options
+#pod are:
+#pod
+#pod =over 4
+#pod
+#pod =item date
+#pod
+#pod The date to build for. Only posts on or before this date will be built.
+#pod Defaults to the current date.
+#pod
+#pod =back
+#pod
+#pod =cut
 
 # sub pages
 around pages => sub {
@@ -359,6 +505,17 @@ around pages => sub {
     return @all_pages;
 };
 
+#pod =method tags
+#pod
+#pod     my @links = $app->tags;
+#pod
+#pod Get a set of L<link objects|Statocles::Link> suitable for creating a list of
+#pod tag links. The common attributes are:
+#pod
+#pod     text => 'The tag text'
+#pod     href => 'The URL to the tag page'
+#pod
+#pod =cut
 
 sub tags {
     my ( $self ) = @_;
@@ -379,6 +536,25 @@ sub _tag_url {
     return $tag;
 }
 
+#pod =method recent_posts
+#pod
+#pod     my @pages = $app->recent_posts( $count, %filter );
+#pod
+#pod Get the last $count recent posts for this blog. Useful for templates and site
+#pod index pages.
+#pod
+#pod %filter is an optional set of filters to apply to only show recent posts
+#pod matching the given criteria. The following filters are available:
+#pod
+#pod =over 4
+#pod
+#pod =item tags
+#pod
+#pod (string) Only show posts with the given tag
+#pod
+#pod =back
+#pod
+#pod =cut
 
 sub recent_posts {
     my ( $self, $count, %filter ) = @_;
@@ -403,6 +579,14 @@ sub recent_posts {
     return @found_pages;
 }
 
+#pod =method page_url
+#pod
+#pod     my $url = $app->page_url( $page )
+#pod
+#pod Return the absolute URL to this L<page object|Statocles::Page>, removing the
+#pod "/index.html" if necessary.
+#pod
+#pod =cut
 
 # XXX This is TERRIBLE. We need to do this better. Perhaps a "url()" helper in the
 # template? And a full_url() helper? Or perhaps the template knows whether it should
@@ -427,7 +611,7 @@ Statocles::App::Blog - A blog application
 
 =head1 VERSION
 
-version 0.070
+version 0.071
 
 =head1 DESCRIPTION
 
