@@ -1,6 +1,7 @@
 package Statocles::Store;
+$Statocles::Store::VERSION = '0.071';
 # ABSTRACT: The source for data documents and files
-$Statocles::Store::VERSION = '0.070';
+
 use Statocles::Base 'Class';
 use Scalar::Util qw( weaken blessed );
 use Statocles::Document;
@@ -13,6 +14,11 @@ use Module::Runtime qw( use_module );
 # path)
 my %FILE_STORES = ();
 
+#pod =attr path
+#pod
+#pod The path to the directory containing the L<documents|Statocles::Document>.
+#pod
+#pod =cut
 
 has path => (
     is => 'ro',
@@ -21,6 +27,12 @@ has path => (
     required => 1,
 );
 
+#pod =attr document_extensions
+#pod
+#pod An array of file extensions that should be considered documents. Defaults to
+#pod "markdown" and "md".
+#pod
+#pod =cut
 
 has document_extensions => (
     is => 'ro',
@@ -35,6 +47,17 @@ has document_extensions => (
     },
 );
 
+#pod =attr documents
+#pod
+#pod All the L<documents|Statocles::Document> currently read by this store.
+#pod
+#pod =method clear
+#pod
+#pod     $store->clear;
+#pod
+#pod Clear the cached documents in this Store.
+#pod
+#pod =cut
 
 has documents => (
     is => 'rw',
@@ -86,6 +109,14 @@ sub DEMOLISH {
     }
 }
 
+#pod =method read_documents
+#pod
+#pod     my $docs = $store->read_documents;
+#pod
+#pod Read the directory C<path> and create the L<document
+#pod objects|Statocles::Document> inside.  Returns an arrayref of document objects.
+#pod
+#pod =cut
 
 sub read_documents {
     my ( $self ) = @_;
@@ -118,16 +149,48 @@ sub _is_owned_path {
     return 1;
 }
 
+#pod =method read_document
+#pod
+#pod     my $doc = $store->read_document( $path )
+#pod
+#pod Read a single L<document|Statocles::Document> in Markdown with optional YAML
+#pod frontmatter.
+#pod
+#pod =cut
 
 sub read_document {
     my ( $self, $path ) = @_;
     site->log->debug( "Read document: " . $path );
     my $full_path = $self->path->child( $path );
-    my %doc = $self->parse_frontmatter( $full_path, $full_path->slurp_utf8 );
+    my $relative_path = $full_path->relative( cwd );
+    my %doc = $self->parse_frontmatter( $relative_path, $full_path->slurp_utf8 );
     my $class = $doc{class} ? use_module( delete $doc{class} ) : 'Statocles::Document';
-    return $class->new( %doc, path => $path, store => $self );
+    my $obj = eval { $class->new( %doc, path => $path, store => $self ) };
+    if ( $@ ) {
+        if ( ref $@ && $@->isa( 'Error::TypeTiny::Assertion' ) ) {
+            die sprintf qq{Error creating document in "%s": Value "%s" is not valid for attribute "%s" (expected "%s")\n},
+                $relative_path,
+                $@->value,
+                $@->attribute_name,
+                $@->type;
+        }
+        else {
+            die sprintf qq{Error creating document in "%s": %s\n},
+                $@;
+        }
+    }
+    return $obj;
 }
 
+#pod =method parse_frontmatter
+#pod
+#pod     my %doc_attrs = $store->parse_frontmatter( $from, $content )
+#pod
+#pod Parse a document with YAML frontmatter. $from is a string identifying where the
+#pod content comes from (a path or other identifier). $content is the content to
+#pod parse for frontmatter.
+#pod
+#pod =cut
 
 sub parse_frontmatter {
     my ( $self, $from, $content ) = @_;
@@ -143,7 +206,7 @@ sub parse_frontmatter {
 
         # If we did not find the marker between YAML and Markdown
         if ( !defined $i ) {
-            die "Could not find end of front matter (---) in '$from'\n";
+            die qq{Could not find end of front matter (---) in "$from"\n};
         }
 
         # Before the marker is YAML
@@ -151,7 +214,7 @@ sub parse_frontmatter {
             $doc = YAML::Load( join "\n", splice( @lines, 0, $i ), "" );
         };
         if ( $@ ) {
-            die "Error parsing YAML in '$from'\n$@";
+            die qq{Error parsing YAML in "$from"\n$@};
         }
 
         # Remove the last '---' mark
@@ -165,14 +228,24 @@ sub parse_frontmatter {
             $doc->{date} = DateTimeObj->assert_coerce( $doc->{date} );
         };
         if ( $@ ) {
-            die sprintf "Could not parse date '%s'. Does not match 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS'",
-                $doc->{date};
+            die sprintf qq{Could not parse date "%s" in "%s": Does not match "YYYY-MM-DD" or "YYYY-MM-DD HH:MM:SS"\n},
+                $doc->{date},
+                $from;
         }
     }
 
     return %$doc;
 }
 
+#pod =method write_document
+#pod
+#pod     $store->write_document( $path, $doc );
+#pod
+#pod Write a L<document|Statocles::Document> to the store at the given store path.
+#pod
+#pod The document is written in Frontmatter format.
+#pod
+#pod =cut
 
 sub write_document {
     my ( $self, $path, $doc ) = @_;
@@ -211,6 +284,13 @@ sub _freeze_document {
     return $doc;
 }
 
+#pod =method is_document
+#pod
+#pod     my $bool = $store->is_document( $path );
+#pod
+#pod Returns true if the path looks like a document path (matches the L</document_extensions>).
+#pod
+#pod =cut
 
 sub is_document {
     my ( $self, $path ) = @_;
@@ -218,6 +298,13 @@ sub is_document {
     return $path =~ /[.](?:$match)$/;
 }
 
+#pod =method read_file
+#pod
+#pod     my $content = $store->read_file( $path )
+#pod
+#pod Read the file from the given C<path>.
+#pod
+#pod =cut
 
 sub read_file {
     my ( $self, $path ) = @_;
@@ -225,12 +312,38 @@ sub read_file {
     return $self->path->child( $path )->slurp_utf8;
 }
 
+#pod =method has_file
+#pod
+#pod     my $bool = $store->has_file( $path )
+#pod
+#pod Returns true if a file exists with the given C<path>.
+#pod
+#pod NOTE: This should not be used to check for directories, as not all stores have
+#pod directories.
+#pod
+#pod =cut
 
 sub has_file {
     my ( $self, $path ) = @_;
     return $self->path->child( $path )->is_file;
 }
 
+#pod =method find_files
+#pod
+#pod     my $iter = $store->find_files( %opt )
+#pod     while ( my $path = $iter->() ) {
+#pod         # ...
+#pod     }
+#pod
+#pod Returns an iterator that, when called, produces a single path suitable to be passed
+#pod to L<read_file>.
+#pod
+#pod Available options are:
+#pod
+#pod     include_documents      - If true, will include files that look like documents.
+#pod                              Defaults to false.
+#pod
+#pod =cut
 
 sub find_files {
     my ( $self, %opt ) = @_;
@@ -249,12 +362,33 @@ sub find_files {
     };
 }
 
+#pod =method open_file
+#pod
+#pod     my $fh = $store->open_file( $path )
+#pod
+#pod Open the file with the given path. Returns a filehandle.
+#pod
+#pod The filehandle opened is using raw bytes, not UTF-8 characters.
+#pod
+#pod =cut
 
 sub open_file {
     my ( $self, $path ) = @_;
     return $self->path->child( $path )->openr_raw;
 }
 
+#pod =method write_file
+#pod
+#pod     $store->write_file( $path, $content );
+#pod
+#pod Write the given C<content> to the given C<path>. This is mostly used to write
+#pod out L<page objects|Statocles::Page>.
+#pod
+#pod C<content> may be a simple string or a filehandle. If given a string, will
+#pod write the string using UTF-8 characters. If given a filehandle, will write out
+#pod the raw bytes read from it with no special encoding.
+#pod
+#pod =cut
 
 sub write_file {
     my ( $self, $path, $content ) = @_;
@@ -277,6 +411,14 @@ sub write_file {
     return;
 }
 
+#pod =method remove
+#pod
+#pod     $store->remove( $path )
+#pod
+#pod Remove the given path from the store. If the path is a directory, the entire
+#pod directory is removed.
+#pod
+#pod =cut
 
 sub remove {
     my ( $self, $path ) = @_;
@@ -296,7 +438,7 @@ Statocles::Store - The source for data documents and files
 
 =head1 VERSION
 
-version 0.070
+version 0.071
 
 =head1 DESCRIPTION
 
